/*
 * A program to manipulate a cube
 * The cube is rotating over its diagonal (0,0,0) to (1,1,1)
 *  cube.c v. 0.0
 */
#include <GL/glut.h>
#include <stdio.h>

float c1[8][3], c2[8][3], c3[8][3], c4[8][3];
float yrot;
float xrot;
float angle;
int windowWidth, windowHeight;

/** Timer **/
int time=500;  // = 0.5 seg
int timerflag = 1;
int signo = 1;

/** 
	Function that generate a cube with center in (cx,cy,cz)
	with size value for its side
**/
void getcube ( float cx, float cy, float cz, float size, float v[8][3] ) 
{
	v[0][0] = v[3][0] = v[4][0] = v[7][0] = cx;
	v[0][1] = v[1][1] = v[2][1] = v[3][1] = cy;
	v[0][2] = v[1][2] = v[4][2] = v[5][2] = cz;
	v[1][0] = v[2][0] = v[5][0] = v[6][0] = cx + size;
	v[4][1] = v[5][1] = v[6][1] = v[7][1] = cy + size;
	v[2][2] = v[3][2] = v[6][2] = v[7][2] = cz + size;
}

void show_cube ( float v[8][3] )
{
	static GLfloat n[6][3] = {
		{ 0.0, 0.0, 1.0},
		{ 1.0, 0.0, 0.0},
		{ 0.0, 0.0,-1.0},
		{-1.0, 0.0, 0.0},
		{ 0.0, 1.0, 0.0},
		{ 0.0,-1.0, 0.0} };

	static GLint faces[6][4] = {
		{7,6,2,3},
		{6,5,1,2},
		{5,1,0,4},
		{4,0,3,7},
		{4,5,6,7},
		{0,1,2,3} };

	static GLfloat colors[6][3] = {
		{ 1.0, 1.0, 1.0}, // white
		{ 1.0, 0.0, 0.0}, // red
		{ 0.0, 1.0, 0.0}, // green
		{ 0.0, 0.0, 1.0}, // blue
		{ 1.0, 1.0, 0.0}, // yellow
		{ 1.0, 0.0, 1.0}  // magenta 
	};

	int i;

	for (i = 5; i >= 0; i--) {
		glBegin(GL_QUADS);
		// glBegin(GL_LINE_LOOP);
		glColor3fv( &colors[i][0] );
		glNormal3fv(&n[i][0]);
		glVertex3fv(&v[faces[i][0]][0]);
		glVertex3fv(&v[faces[i][1]][0]);
		glVertex3fv(&v[faces[i][2]][0]);
		glVertex3fv(&v[faces[i][3]][0]);
		glEnd();
	}
}

void display(void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */
	glRotatef (xrot, 1.0, 0.0, 0.0);
	glRotatef (yrot, 0.0, 1.0, 0.0);

	/* viewing transformation  */
	glPushMatrix( );
	glScalef ( 0.2, 0.2, 0.2);      /* modeling transformation */ 

	show_cube( c1 );
	show_cube( c2 );

	glPushMatrix( );
	glRotatef ( angle , 1.0, 0.0, 0.0);
	show_cube( c3 );
	show_cube( c4 );
	glPopMatrix( );

	// Draw x axis
	glBegin(GL_LINES);
		glColor3f( 1.0, 0.0, 0.0 );
		glVertex3f(-0.5, 0.0, 0.0);
		glVertex3f( 3.5, 0.0, 0.0);
	glEnd();
	
	glPopMatrix( );

	glutSwapBuffers();
}

void init(void) 
{
	glClearColor (0.8, 0.8, 0.8, 0.0);	// Background to white
	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0,-1.0, 1.0,-1.0, 1.0);
	// glShadeModel (GL_FLAT);
	getcube ( 0.0,-0.5,-1.5, 1.0, c1 );
	getcube ( 0.0,-0.5, 0.5, 1.0, c2 );
	getcube ( 2.0,-0.5,-1.5, 1.0, c3 );
	getcube ( 2.0,-0.5, 0.5, 1.0, c4 );

	glEnable (GL_DEPTH_TEST);
}

/** 
	w=WindowWidth, h=WindowHeight   
**/
void reshape (int w, int h)
{
	float aspect = (float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glOrtho (-aspect, aspect, -1.0, 1.0, -1.0, 1.0);
}

static void mouse (int x, int y)
{
	yrot = (float)x*360.f/(float)windowWidth - 180.f;
	xrot = (float)y*360.f/(float)windowHeight - 180.f;
	glutPostRedisplay ();
}

static void Timer( int val )
{
	// static int i=0;

	if ( !val ) return;

	// fprintf( stderr, "Hola %d\n", i );
	// i++;

	angle += 10.0*signo;
	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;

	glutTimerFunc( time, Timer, timerflag );
	// rota_cubo( c1 );
	glutPostRedisplay( );
}

/**
	Keyboard callback function 
**/ 
void keyboard(unsigned char key, int x, int y)
{
	switch (key) {
		case 27:
			exit(0);
			break;
		case 's':   // stop animation
			timerflag = 0;
			break;
		case 'i':   // start animation
			timerflag =  1;
			glutTimerFunc( time, Timer, timerflag );
			break;
		case '+':   // increase animation elapsed time
			time +=  20;
			break;
		case '-':   // decrease animation elapsed time
			time -=  20;
			if ( time<20 ) time = 20;
			break;
		case 'h':   // Positive rotation
			signo = 1;
			break;
		case 'j':   // Positive rotation
			signo = -1;
			break;
		default:
			break;
	}
}

int main(int argc, char *argv[] )
{
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	glutInitWindowSize (windowWidth=400, windowHeight=400); 
	glutInitWindowPosition (100, 100);
	glutCreateWindow (argv[0]);
	init ();
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutMotionFunc (mouse);
	glutKeyboardFunc(keyboard);
	glutTimerFunc( time, Timer, timerflag );

	glutMainLoop();
	return 0;
}
